
package org.mule.modules.zuora.config.spring;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.modules.zuora.config.SubscribeMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class SubscribeDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public SubscribeDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(SubscribeMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", new RuntimeBeanReference(configRef));
        }
        if ((element.getAttribute("session-ref")!= null)&&(!StringUtils.isBlank(element.getAttribute("session-ref")))) {
            builder.addPropertyValue("session", new RuntimeBeanReference(element.getAttribute("session-ref")));
        }
        Element subscriptionsListElement = null;
        subscriptionsListElement = DomUtils.getChildElementByTagName(element, "subscriptions");
        List<Element> subscriptionsListChilds = null;
        if (subscriptionsListElement!= null) {
            String subscriptionsRef = subscriptionsListElement.getAttribute("ref");
            if ((subscriptionsRef!= null)&&(!StringUtils.isBlank(subscriptionsRef))) {
                if ((!subscriptionsRef.startsWith(patternInfo.getPrefix()))&&(!subscriptionsRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("subscriptions", new RuntimeBeanReference(subscriptionsRef));
                } else {
                    builder.addPropertyValue("subscriptions", subscriptionsRef);
                }
            } else {
                ManagedList subscriptions = new ManagedList();
                subscriptionsListChilds = DomUtils.getChildElementsByTagName(subscriptionsListElement, "subscription");
                if (subscriptionsListChilds!= null) {
                    for (Element subscriptionsChild: subscriptionsListChilds) {
                        String valueRef = subscriptionsChild.getAttribute("value-ref");
                        if ((valueRef!= null)&&(!StringUtils.isBlank(valueRef))) {
                            subscriptions.add(new RuntimeBeanReference(valueRef));
                        } else {
                            subscriptions.add(subscriptionsChild.getTextContent());
                        }
                    }
                }
                builder.addPropertyValue("subscriptions", subscriptions);
            }
        }
        if ((element.getAttribute("username")!= null)&&(!StringUtils.isBlank(element.getAttribute("username")))) {
            builder.addPropertyValue("username", element.getAttribute("username"));
        }
        if ((element.getAttribute("password")!= null)&&(!StringUtils.isBlank(element.getAttribute("password")))) {
            builder.addPropertyValue("password", element.getAttribute("password"));
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
            if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                propertyValues.addPropertyValue("messageProcessors", new ManagedList());
            }
            List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
            listMessageProcessors.add(definition);
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
