
package org.mule.modules.zuora.config.spring;

import org.apache.commons.lang.StringUtils;
import org.mule.api.lifecycle.Disposable;
import org.mule.api.lifecycle.Initialisable;
import org.mule.config.PoolingProfile;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.parsers.generic.AutoIdUtils;
import org.mule.modules.zuora.config.ZuoraModuleSessionManagerAdapter;
import org.mule.util.TemplateParser;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class ZuoraModuleConfigDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public ZuoraModuleConfigDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        String name = element.getAttribute("name");
        if ((name == null)||StringUtils.isBlank(name)) {
            element.setAttribute("name", AutoIdUtils.getUniqueName(element, "mule-bean"));
        }
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(ZuoraModuleSessionManagerAdapter.class.getName());
        if (Initialisable.class.isAssignableFrom(ZuoraModuleSessionManagerAdapter.class)) {
            builder.setInitMethodName(Initialisable.PHASE_NAME);
        }
        if (Disposable.class.isAssignableFrom(ZuoraModuleSessionManagerAdapter.class)) {
            builder.setDestroyMethodName(Disposable.PHASE_NAME);
        }
        if ((element.getAttribute("client-ref")!= null)&&(!StringUtils.isBlank(element.getAttribute("client-ref")))) {
            builder.addPropertyValue("client", new RuntimeBeanReference(element.getAttribute("client-ref")));
        }
        if ((element.getAttribute("endpoint")!= null)&&(!StringUtils.isBlank(element.getAttribute("endpoint")))) {
            builder.addPropertyValue("endpoint", element.getAttribute("endpoint"));
        }
        if ((element.getAttribute("username")!= null)&&(!StringUtils.isBlank(element.getAttribute("username")))) {
            builder.addPropertyValue("username", element.getAttribute("username"));
        }
        if ((element.getAttribute("password")!= null)&&(!StringUtils.isBlank(element.getAttribute("password")))) {
            builder.addPropertyValue("password", element.getAttribute("password"));
        }
        BeanDefinitionBuilder sessionPoolingProfileBuilder = BeanDefinitionBuilder.rootBeanDefinition(PoolingProfile.class.getName());
        Element sessionPoolingProfileElement = DomUtils.getChildElementByTagName(element, "session-pooling-profile");
        if (sessionPoolingProfileElement!= null) {
            if ((sessionPoolingProfileElement.getAttribute("maxActive")!= null)&&(!StringUtils.isBlank(sessionPoolingProfileElement.getAttribute("maxActive")))) {
                sessionPoolingProfileBuilder.addPropertyValue("maxActive", sessionPoolingProfileElement.getAttribute("maxActive"));
            }
            if ((sessionPoolingProfileElement.getAttribute("maxIdle")!= null)&&(!StringUtils.isBlank(sessionPoolingProfileElement.getAttribute("maxIdle")))) {
                sessionPoolingProfileBuilder.addPropertyValue("maxIdle", sessionPoolingProfileElement.getAttribute("maxIdle"));
            }
            if ((sessionPoolingProfileElement.getAttribute("maxWait")!= null)&&(!StringUtils.isBlank(sessionPoolingProfileElement.getAttribute("maxWait")))) {
                sessionPoolingProfileBuilder.addPropertyValue("maxWait", sessionPoolingProfileElement.getAttribute("maxWait"));
            }
            if ((sessionPoolingProfileElement.getAttribute("exhaustedAction")!= null)&&(!StringUtils.isBlank(sessionPoolingProfileElement.getAttribute("exhaustedAction")))) {
                sessionPoolingProfileBuilder.addPropertyValue("exhaustedAction", PoolingProfile.POOL_EXHAUSTED_ACTIONS.get(sessionPoolingProfileElement.getAttribute("exhaustedAction")));
            }
            if ((sessionPoolingProfileElement.getAttribute("exhaustedAction")!= null)&&(!StringUtils.isBlank(sessionPoolingProfileElement.getAttribute("exhaustedAction")))) {
                sessionPoolingProfileBuilder.addPropertyValue("initialisationPolicy", PoolingProfile.POOL_INITIALISATION_POLICIES.get(sessionPoolingProfileElement.getAttribute("initialisationPolicy")));
            }
            builder.addPropertyValue("sessionPoolingProfile", sessionPoolingProfileBuilder.getBeanDefinition());
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        return definition;
    }

}
